@@ -1,6 +1,5 @@
 import csv
 import logging
-import sqlite3
 import sys
 from collections import defaultdict
 from pathlib import Path
@@ -8,6 +7,7 @@ from pprint import pformat
 
 import yaml
 
+from .sqlite_db import SqliteDB
 from .utils import expand_paths, colored, Pool, cpu_count, yield_xml_doc
 from .xmldoctotabular import XmlDocToTabular
 
@@ -17,11 +17,11 @@ class XmlCollectionToTabular:
         self,
         xml_input,
         config,
-        dtd_path,
         output_path,
-        output_type,
-        log_level=logging.INFO,
+        output_type="sqlite",
+        dtd_path=None,
         preprocess_doc=None,
+        log_level=logging.INFO,
         **kwargs,
     ):
 
@@ -30,6 +30,8 @@ class XmlCollectionToTabular:
         self.logger.addHandler(logging.StreamHandler(sys.stdout))
 
         self.xml_files = []
+        if isinstance(xml_input, str):
+            xml_input = [xml_input]
         for input_path in xml_input:
             for path in expand_paths(input_path):
                 if path.is_file():
@@ -42,16 +44,34 @@ class XmlCollectionToTabular:
                     self.logger.fatal("specified input is invalid")
                     exit(1)
 
-        # do this now, because we don't want to process all that data and then find
-        #  the output_path is invalid... :)
-        self.output_path = Path(output_path)
-        self.output_path.mkdir(parents=True, exist_ok=True)
-
-        self.config = yaml.safe_load(open(config))
+        self.config = config
+        if isinstance(config, str) and Path(config).is_file():
+            self.config = yaml.safe_load(open(config))
 
         self.output_type = output_type
+        self.output_path = Path(output_path).resolve()
+
         if self.output_type == "sqlite":
-            self.init_sqlite_db()
+            if self.output_path.is_dir():
+                self.output_path = (self.output_path / "db.sqlite").resolve()
+
+            if self.output_path.suffix != "sqlite":
+                self.output_path = self.output_path.with_suffix(".sqlite")
+
+            if self.output_path.exists():
+                self.logger.warning(
+                    colored(
+                        "Sqlite database %s exists; records will be appended.",
+                        "yellow",
+                    ),
+                    self.output_path,
+                )
+            else:
+                self.output_path.parent.mkdir(parents=True, exist_ok=True)
+
+            self.init_sqlite_db(self.output_path)
+        else:
+            self.output_path.mkdir(parents=True, exist_ok=True)
 
         self.dtd_path = dtd_path
         self.preprocess_doc = preprocess_doc
@@ -59,7 +79,7 @@ class XmlCollectionToTabular:
         self.processes = kwargs["processes"]
         self.continue_on_error = kwargs["continue_on_error"]
 
-        self.fieldnames = self.get_fieldnames()
+        self.fieldnames = self.get_fieldnames(self.config)
         self.set_root_config()
 
     def set_root_config(self):
@@ -73,33 +93,16 @@ class XmlCollectionToTabular:
                 self.config["xml_root"],
             )
 
-    def init_sqlite_db(self):
-        try:
-            from sqlite_utils import Database as SqliteDB  # noqa
+    def init_sqlite_db(self, output_path):
+        self.db = SqliteDB(output_path)
 
-        except ImportError:
-            self.logger.debug("sqlite_utils (pip3 install sqlite-utils) not available")
-            raise
+        self.db.execute("pragma synchronous=off;")
+        self.db.execute("pragma journal_mode=memory;")
 
-        self.db_path = (self.output_path / "db.sqlite").resolve()
-        if self.db_path.exists():
-            self.logger.warning(
-                colored(
-                    "Sqlite database %s exists; records will be appended.",
-                    "yellow",
-                ),
-                self.db_path,
-            )
-
-        db_conn = sqlite3.connect(str(self.db_path), isolation_level=None)
-        db_conn.execute("pragma synchronous=off;")
-        db_conn.execute("pragma journal_mode=memory;")
-        self.db = SqliteDB(db_conn)
-
-        for tablename, fieldnames in self.get_fieldnames().items():
+        for tablename, fieldnames in self.get_fieldnames(self.config).items():
             if tablename in self.db.table_names():
                 for fieldname in fieldnames:
-                    if fieldname not in self.db[tablename].columns_dict:
+                    if fieldname not in self.db[tablename].columns:
                         self.db[tablename].add_column(fieldname, str)
                 continue
             params = {"column_order": fieldnames}
@@ -158,6 +161,9 @@ class XmlCollectionToTabular:
                     colored("No records found! (config file error?)", "red")
                 )
 
+        if self.output_type == "sqlite" and self.output_path == ":memory:":
+            return self.db
+
     def flush_to_disk(self, tables):
         if self.output_type == "csv":
             self.write_csv_files(tables)
@@ -165,7 +171,8 @@ class XmlCollectionToTabular:
         if self.output_type == "sqlite":
             self.write_sqlitedb(tables)
 
-    def get_fieldnames(self):
+    @staticmethod
+    def get_fieldnames(full_config):
         # On python >=3.7, dictionaries maintain key order, so fields are guaranteed to
         #  be returned in the order in which they appear in the config file.  To
         #  guarantee this on versions of python <3.7 (insofar as it matters),
@@ -215,7 +222,7 @@ class XmlCollectionToTabular:
                 + "\n ".join(pformat(config).split("\n"))
             )
 
-        for key, config in self.config.items():
+        for key, config in full_config.items():
             if key.startswith("<"):
                 # skip keyword instructions
                 continue
@@ -252,7 +259,7 @@ class XmlCollectionToTabular:
                     writer.writerows(rows)
 
     def write_sqlitedb(self, tables):
-        self.logger.info(colored("Writing records to %s ...", "green"), self.db_path)
+        self.logger.info(colored("Writing records to %s ...", "green"), self.output_path)
         self.db.conn.execute("begin exclusive;")
         for tablename, rows in tables.items():
             self.logger.info(
